@@ -23,6 +23,7 @@ TA_sfnt_compute_global_hints(SFNT* sfnt,
   FT_Error error;
   FT_Face face = sfnt->face;
   FT_UInt idx;
+  FT_Int32 load_flags;
 
 
   error = ta_loader_init(font->loader);
@@ -31,7 +32,16 @@ TA_sfnt_compute_global_hints(SFNT* sfnt,
 
   error = FT_Select_Charmap(face, FT_ENCODING_UNICODE);
   if (error)
-    return TA_Err_Missing_Unicode_CMap;
+  {
+    if (font->symbol)
+    {
+      error = FT_Select_Charmap(face, FT_ENCODING_MS_SYMBOL);
+      if (error)
+        return TA_Err_Missing_Symbol_CMap;
+    }
+    else
+      return TA_Err_Missing_Unicode_CMap;
+  }
 
   if (font->symbol)
     idx = 0;
@@ -45,8 +55,9 @@ TA_sfnt_compute_global_hints(SFNT* sfnt,
       return TA_Err_Missing_Glyph;
   }
 
-  error = ta_loader_load_glyph(font->loader, face, idx,
-                               font->fallback_script << 30);
+  load_flags = font->fallback_script << 30;
+  load_flags |= 1 << 29; /* vertical hinting only */
+  error = ta_loader_load_glyph(font->loader, face, idx, load_flags);
 
   return error;
 }
