@@ -27,7 +27,7 @@
 
 
 #ifdef TA_CONFIG_OPTION_USE_WARPER
-#include "afwarp.h"
+#include "tawarp.h"
 #endif
 
 
@@ -122,7 +122,10 @@ ta_latin_metrics_init_widths(TA_LatinMetrics metrics,
         }
       }
 
-      ta_sort_widths(num_widths, axis->widths);
+      /* this also replaces multiple almost identical stem widths */
+      /* with a single one (the value 100 is heuristic) */
+      ta_sort_and_quantize_widths(&num_widths, axis->widths,
+                                  dummy->units_per_em / 100);
       axis->width_count = num_widths;
     }
 
@@ -185,8 +188,8 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
   /* `ta_latin_blue_chars[blues]' string, then finding its top-most or */
   /* bottom-most points (depending on `TA_IS_TOP_BLUE') */
 
-  TA_LOG(("blue zones computation\n"));
-  TA_LOG(("------------------------------------------------\n"));
+  TA_LOG(("blue zones computation\n"
+          "======================\n\n"));
 
   for (bb = 0; bb < TA_LATIN_BLUE_MAX; bb++)
   {
@@ -196,7 +199,7 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
     FT_Pos* blue_shoot;
 
 
-    TA_LOG(("blue %3d: ", bb));
+    TA_LOG(("blue zone %d:\n", bb));
 
     num_flats = 0;
     num_rounds = 0;
@@ -210,8 +213,6 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
       FT_Bool round = 0;
 
 
-      TA_LOG(("'%c'", *p));
-
       /* load the character in the face -- skip unknown or empty ones */
       glyph_index = FT_Get_Char_Index(face, (FT_UInt)*p);
       if (glyph_index == 0)
@@ -275,7 +276,7 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
             best_last = last;
           }
         }
-        TA_LOG(("%5ld", best_y));
+        TA_LOG(("  %c  %ld", *p, best_y));
       }
 
       /* now check whether the point belongs to a straight or round */
@@ -283,6 +284,7 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
       /* lies, then inspect its previous and next points */
       if (best_point >= 0)
       {
+        FT_Pos best_x = points[best_point].x;
         FT_Int prev, next;
         FT_Pos dist;
 
@@ -299,9 +301,12 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
           else
             prev = best_last;
 
-          dist = points[prev].y - best_y;
-          if (dist < -5 || dist > 5)
-            break;
+          dist = TA_ABS(points[prev].y - best_y);
+          /* accept a small distance or a small angle (both values are */
+          /* heuristic; value 20 corresponds to approx. 2.9 degrees)   */
+          if (dist > 5)
+            if (TA_ABS(points[prev].x - best_x) <= 20 * dist)
+              break;
         } while (prev != best_point);
 
         do
@@ -311,9 +316,10 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
           else
             next = best_first;
 
-          dist = points[next].y - best_y;
-          if (dist < -5 || dist > 5)
-            break;
+          dist = TA_ABS(points[next].y - best_y);
+          if (dist > 5)
+            if (TA_ABS(points[next].x - best_x) <= 20 * dist)
+              break;
         } while (next != best_point);
 
         /* now set the `round' flag depending on the segment's kind */
@@ -321,7 +327,7 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
           FT_CURVE_TAG(glyph->outline.tags[prev]) != FT_CURVE_TAG_ON
           || FT_CURVE_TAG(glyph->outline.tags[next]) != FT_CURVE_TAG_ON);
 
-        TA_LOG(("%c ", round ? 'r' : 'f'));
+        TA_LOG((" (%s)\n", round ? "round" : "flat"));
       }
 
       if (round)
@@ -330,13 +336,11 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
         flats[num_flats++] = best_y;
     }
 
-    TA_LOG(("\n"));
-
     if (num_flats == 0 && num_rounds == 0)
     {
       /* we couldn't find a single glyph to compute this blue zone, */
       /* we will simply ignore it then */
-      TA_LOG(("empty\n"));
+      TA_LOG(("  empty\n"));
       continue;
     }
 
@@ -379,8 +383,13 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
 
 
       if (TA_LATIN_IS_TOP_BLUE(bb) ^ over_ref)
+      {
         *blue_ref =
         *blue_shoot = (shoot + ref) / 2;
+
+        TA_LOG(("  [overshoot smaller than reference,"
+                " taking mean value]\n"));
+      }
     }
 
     blue->flags = 0;
@@ -393,7 +402,9 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
     if (bb == TA_LATIN_BLUE_SMALL_TOP)
       blue->flags |= TA_LATIN_BLUE_ADJUSTMENT;
 
-    TA_LOG(("-- ref = %ld, shoot = %ld\n", *blue_ref, *blue_shoot));
+    TA_LOG(("    -> reference = %ld\n"
+            "       overshoot = %ld\n",
+            *blue_ref, *blue_shoot));
   }
 
   TA_LOG(("\n"));
